@@ -141,6 +141,15 @@ def test_imagemask_colorspace(trivial):
     assert pdfimage.colorspace is None
 
 
+def test_imagemask_decode(trivial):
+    xobj, _ = trivial
+    rawimage = xobj
+    rawimage.ImageMask = True
+    pdfimage = PdfImage(rawimage)
+    assert pdfimage.image_mask
+    assert pdfimage._decode_array == (0.0, 1.0)
+
+
 def test_malformed_palette(trivial):
     xobj, _ = trivial
     rawimage = xobj
@@ -689,6 +698,19 @@ def test_stacked_compression(first_image_in):
     assert pim.filters == ['/FlateDecode', '/JPXDecode']
 
 
+def test_stacked_compression_no_orphaned_objects(first_image_in):
+    # Regression test for issue #691
+    xobj, pdf = first_image_in('pike-flate-jp2.pdf')
+
+    initial_count = len(pdf.objects)
+    pim = PdfImage(xobj)
+
+    for _ in range(3):
+        pim.as_pil_image().close()
+
+    assert len(pdf.objects) == initial_count
+
+
 @pytest.mark.parametrize(
     'blackis1,decode,expected',
     [
@@ -1044,7 +1066,10 @@ def test_grayscale_stride(spec):
     im = Image.open(bio)
     assert im.mode == 'L' and im.size == pim.size
 
-    for n, pixel in enumerate(im.getdata()):
+    imdata = (
+        im.get_flattened_data() if hasattr(im, 'get_flattened_data') else im.getdata()
+    )
+    for n, pixel in enumerate(imdata):
         idx = stride * (n // w) + (n % w)
         assert imdata_unpacked[idx] == pixel
 
